#!/bin/sh

PIDFILE=/var/run/update-code.pid
if [ -e ${PIDFILE} ] && kill -0 `cat ${PIDFILE}` 2>/dev/null ; then
	echo "already running"
	exit
fi
echo $$ > ${PIDFILE}
trap "rm -f ${PIDFILE}; exit" INT TERM EXIT

source /etc/ng-ap.conf

source $CONFIG/api.conf
source $SCRIPT/utils/misc.sh
source $SCRIPT/utils/token.sh

fw=/tmp/sysupgrade.bin

clean() {
  rm -rf $fw
}

failed() {
  clean

  echo "failed $1"
  local reason=`encodeURIComponent "$1"`

  wget -T 5 -qO /dev/null $(getAuthHeader) "$CLOUD_ADDRESS/failed?type=code&reason=$reason"
  exit
}

echo "Update code"
echo -n "  Checking for code update... "

# download fw
version=`cat /etc/installed 2>/dev/null`
wget -T 5 -qO $fw $(getAuthHeader) "$CLOUD_ADDRESS/code/download?version=${version}"
[ ! -s $fw ] && echo "up-to-date" && clean && exit
echo "found"

# verify checksum
echo -n "  Verifing checksum... "
checksum=`md5 $fw`
wget -T 5 -qO /dev/null $(getAuthHeader) "$CLOUD_ADDRESS/code/verify?algo=md5&checksum=$checksum"
[ $? -ne 0 ] && failed "checksum"
echo "matched"

# extract metadata
echo -n "  Extracting firmware metadata... "
fwtool -q -i /tmp/sysupgrade.meta $fw || failed "no meta"
echo "done"

# verify metadata
echo -n "  Verifying firmware metadata... "
source /usr/share/libubox/jshn.sh
json_load "$(cat /tmp/sysupgrade.meta)" || failed "invalid meta"
json_select "version" || failed "invalid meta"
json_get_var dist dist || failed "invalid meta"
json_get_var version version || failed "invalid meta"
json_get_var ngap "ng-ap" || failed "invalid meta"
echo "done"

# upgrade
echo "  Upgrading to $dist version $version, ng-ap version $ngap..."
touch /tmp/installing
sleep 30 && touch /etc/banner && sysupgrade $fw
